#!/home/ywan/soft/anaconda3/bin/python3.7
import math,os
from numpy import mat
import numpy as np

#POSCAR2position
def pos2position(name):
    poscar = open(name)
    line = poscar.readlines()
    lattice = open('lattice','w')
    for i in range(2,5):
        lattice.write(line[i])
    poscar.close()
    lattice.close()
    box = np.loadtxt('lattice')
    coordinate = open('position','w')
    for i in range(8,len(line)):
        coordinate.write(line[i])
    coordinate.close()
    position = np.loadtxt('position')
    atom = line[5]
    atomnum = line[6]
    return box, position,atom,atomnum

#prepare fragments
#step1: backbone
def backbone(backbone):
    box_bb,position_bb,atom_bb,atomnum_bb = pos2position('./backbone/POSCAR-%s' % backbone)
    return box_bb,position_bb,atom_bb,atomnum_bb

#step2: alkyl chain based on backbone
def alkylchaintransformation(name1):
    box_alkyl_t,position_alkyl_t,atom_alkyl_t,atomnum_alkyl_t = pos2position('./template/alkyl/POSCAR-C10-%s' % name1)
    #matbox_alkyl_t = np.matrix(box_alkyl_t)
    position_alkyl_t_ref = np.matrix(position_alkyl_t[:3])
    position_alkyl_t_atom = np.matrix(position_alkyl_t[3:])
    mat_transform = np.dot(position_alkyl_t_atom,position_alkyl_t_ref.I)
    #matbox_bb = np.matrix(box_bb)
    if name1 == 'L':
        position_bb_ref = np.matrix(position_bb[:3])
    else: position_bb_ref = np.matrix(position_bb[3:6])
    position_alkyl = np.dot(mat_transform,position_bb_ref)
    position_alkyl = np.array(position_alkyl)
    position_alkyl_C = position_alkyl[:20]
    position_alkyl_H = position_alkyl[20:]
    return position_alkyl_C, position_alkyl_H

def alkylchainLength(num):
    dic = {}
    dic['C'] = num*4
    dic['H'] = num*8
    position_alkyl_C_L_p = position_alkyl_C_L[:num*2]
    position_alkyl_H_L_p = position_alkyl_H_L[:num*4]
    position_alkyl_C_R_p = position_alkyl_C_R[:num*2]
    position_alkyl_H_R_p = position_alkyl_H_R[:num*4]
    return dic,position_alkyl_C_L_p,position_alkyl_H_L_p,position_alkyl_C_R_p,position_alkyl_H_R_p

#step3: ionic functionality
def functransform(name):
    dicfunc = {'LR':-1,'LL':-2,'RL':-1,'RR':-2}
    box_func,position_func,atom_func,atomnum_func = pos2position('./func/POSCAR-%s' % name)
    matbox_func = np.matrix(box_func)
    matbox_bb = np.matrix(box_bb)
    matbox = np.dot(matbox_func.I,matbox_bb)
    position_func_new = np.dot(position_func,matbox.I)
    position_func_new = np.array(position_func_new)
    vector = position_func_new[1] - position_func_new[0]
    if name[-2:-1] == 'L':
        vector_old = position_func_new[1] - position_alkyl_C_L_p[dicfunc[name[-2:]]]
    else:
         vector_old = position_func_new[1] - position_alkyl_C_R_p[dicfunc[name[-2:]]]
    vector_trans = vector - vector_old
    vector_trans = np.array(vector_trans)
    vector_move = []
    for i in range(len(position_func_new)-1):
        vector_move.append(vector_trans)
    vector_move = np.array(vector_move)
    position_func_new = position_func_new[1:] + vector_move
    atom_func = atom_func.split()[1:]
    atomnum_func = atomnum_func.split()[1:]
    return position_func_new,atom_func,atomnum_func

def POSCAR():
    POSCAR = open('POSCAR','w')
    def writePOSCAR(name):
        for i in range(len(name)):
            for m in name[i]:
                POSCAR.write(str(m) + ' ')
            POSCAR.write('\n')
    POSCAR.write('POSCAR\n' + '1.00\n')
    writePOSCAR(box_bb) 
    POSCAR.write(atom_bb[:-1] + ' ' + ' C H ' + ' ')
    def write(name):
        for i in name:
            POSCAR.write(i + ' ')
    write(atom_func_LL)
    write(atom_func_LR)
    write(atom_func_RL)
    write(atom_func_RR)
    POSCAR.write('\n')
    POSCAR.write(atomnum_bb[:-1] + ' ' + str(dic['C']) + ' ' + str(dic['H']) + ' ')
    write(atomnum_func_LL)
    write(atomnum_func_LR)
    write(atomnum_func_RL)
    write(atomnum_func_RR)
    POSCAR.write('\n' + 'Direct\n')
    writePOSCAR(position_bb) 
    writePOSCAR(position_alkyl_C_L_p) 
    writePOSCAR(position_alkyl_C_R_p) 
    writePOSCAR(position_alkyl_H_L_p) 
    writePOSCAR(position_alkyl_H_R_p) 
    writePOSCAR(position_func_new_LL) 
    writePOSCAR(position_func_new_LR) 
    writePOSCAR(position_func_new_RL) 
    writePOSCAR(position_func_new_RR) 
    POSCAR.close()

#backbone_list = ['CPDT-BBT','CPDT-BT','CPDT-FBT','CPDT-Ph','CPDT-PhCN','CPDT-PhF','CPDT-PT','CPDT-Py','CPDT-Tp']
backbone_list = ['PDT-BBT','PDT-BT','PDT-FBT','PDT-Ph','PDT-PhCN','PDT-PhF','PDT-PT','PDT-Py','PDT-Tp']
#backbone_list = ['CPDT-BBT','CPDT-BT','CPDT-FBT','CPDT-Ph','CPDT-PhCN','CPDT-PhF','CPDT-PT','CPDT-Py','CPDT-Tp']
#backbone_list = ['CPDT-BT']
functionality = ['COOK','SO3K','PO3HK']
for backbone_name in backbone_list:
    box_bb,position_bb,atom_bb,atomnum_bb = backbone(backbone_name)
    position_alkyl_C_L, position_alkyl_H_L = alkylchaintransformation('L')
    position_alkyl_C_R, position_alkyl_H_R = alkylchaintransformation('R')
    for i in [2,4,6,8]:
        dic,position_alkyl_C_L_p,position_alkyl_H_L_p,position_alkyl_C_R_p,position_alkyl_H_R_p = alkylchainLength(i)
        for func_name in functionality:
            position_func_new_LL,atom_func_LL,atomnum_func_LL=functransform('%s-LL' % func_name)
            position_func_new_LR,atom_func_LR,atomnum_func_LR=functransform('%s-LR' % func_name)
            position_func_new_RL,atom_func_RL,atomnum_func_RL=functransform('%s-RL' % func_name)
            position_func_new_RR,atom_func_RR,atomnum_func_RR=functransform('%s-RR' % func_name)
            POSCAR()
            name = backbone_name + '-' + 'C%d' % i + '-' + func_name
            os.system('mv POSCAR POSCAR-%s' % name)



